/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GuiFullRunningJonasFieldFromDiskFrameDescriptor.java 20828 2011-02-03 10:12:20Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.guifullstarter;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;

import javax.swing.JFileChooser;

import org.ow2.jonas.autostart.frame.RunningJonasFieldFromDiskFrame;
import org.ow2.jonas.autostart.fullwizard.FullWizardPanelDescriptor;
import org.ow2.jonas.autostart.starterfull.FullStarter;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class instanciates RunningJonasFieldFromDiskFrame.
 * @author skabore
 */
public class GuiFullRunningJonasFieldFromDiskFrameDescriptor extends FullWizardPanelDescriptor implements ActionListener,
        KeyListener {

    /**
     * The identifier.
     */
    public static final String IDENTIFIER = "RUN_JONAS_FROM_DISK_PANEL_FULL_STARTER";

    /**
     * The RunningJonasFieldFromDiskFrame.
     */
    private RunningJonasFieldFromDiskFrame runningJonasFieldFromDiskFrame;

    /**
     * The starter librairy.
     */
    private FullStarter starter;

    /**
     * The default constructor.
     * @param starter the starter librairy
     * @param steps, the required steps to create a starter
     * @param stepId, the sted number
     */
    public GuiFullRunningJonasFieldFromDiskFrameDescriptor(final FullStarter starter, final int steps, final int stepId) {
        this.starter = starter;
        this.runningJonasFieldFromDiskFrame = new RunningJonasFieldFromDiskFrame(steps, stepId);
        this.runningJonasFieldFromDiskFrame.addActionListener(this);
        this.runningJonasFieldFromDiskFrame.addKeyListener(this);
        setPanelDescriptorIdentifier(IDENTIFIER);
        setPanelComponent(this.runningJonasFieldFromDiskFrame);
    }

    /**
     * @return starter.
     */
    public FullStarter getStarter() {
        return this.starter;
    }

    /**
     * @return the next panel.
     */
    @Override
    public Object getNextPanelDescriptor() {
        getWizard().getStopButton().setVisible(false);
        getWizard().setCurrentFrame(11);
        return GuiFullApplicationFrameDescriptor.IDENTIFIER;
    }

    /**
     * @return the back panel.
     */
    @Override
    public Object getBackPanelDescriptor() {
        getWizard().getStopButton().setVisible(false);
        getWizard().getConfigButton().setVisible(false);
        return GuiFullRunningJonasOptionFrameDescriptor.IDENTIFIER;
    }

    /**
     * @return the config panel.
     */
    @Override
    public Object getConfigPanelDescriptor() {
        getWizard().getStopButton().setVisible(false);
        getWizard().getConfigButton().setVisible(false);
        return "ConfiguratorFrameDescriptor.IDENTIFIER";
    }

    /**
     * Check if the next button must be enabled.
     */
    @Override
    public void aboutToDisplayPanel() {
        setNextButtonAccordingToJonasRootField();
    }

    /**
     * Initialize JOnAS Root (Base) folder when the panel is displaying.
     */
    @Override
    public void displayingPanel() {
        getWizard().setEnabled(false);
        /*
         * new Thread() {
         * @Override public void run() {
         */
        /*
         * showMessage("Please wait while program searches" +
         * " a valid JOnAS folder."); initializeJonasFolder();
         */
        /*
         * } }.start();
         */
        getWizard().setEnabled(true);
        getWizard().getNextButton().setText("Next");
    }

    /**
     * Set the next button enabled according fields values.
     */
    private void setNextButtonAccordingToJonasRootField() {
        if (!checkJavaHome()) {
            getWizard().setNextFinishButtonEnabled(false);
            getWizard().getConfigButton().setVisible(false);
            this.runningJonasFieldFromDiskFrame.getErrorLabel().setText("" + "JAVA_HOME folder seems invalid.");
        } else {
            File javaHome = new File(this.runningJonasFieldFromDiskFrame.getJavaHome().getText());
            this.starter.setJavaHome(javaHome);
            this.runningJonasFieldFromDiskFrame.getErrorLabel().setText("");
            if (Utility.isValidJOnASRootFolder(new File(this.runningJonasFieldFromDiskFrame.getJonasRoot().getText()))) {
                File jRoot = new File(this.runningJonasFieldFromDiskFrame.getJonasRoot().getText());
                this.starter.setJonasRoot(jRoot);
                getWizard().setNextFinishButtonEnabled(true);
                // getWizard().getNextButton().setText("Start");
                getWizard().getNextButton().setMnemonic(KeyEvent.VK_A);
                getWizard().getConfigButton().setVisible(false);
                this.runningJonasFieldFromDiskFrame.getErrorLabel().setText("");
                this.starter.setUseEmbeddedJonas(false);
                disabledConfigButtonAccordingToJonasBaseField();
            } else {
                if (!this.runningJonasFieldFromDiskFrame.getJonasRoot().getText().equals("")) {
                    this.runningJonasFieldFromDiskFrame.getErrorLabel().setText("Error :" + " JOnAS Root folder is not valid.");
                }
                getWizard().setNextFinishButtonEnabled(false);
                getWizard().getConfigButton().setVisible(false);
            }
        }
    }

    /**
     * Disable config button according JOnAS Base fields.
     */
    private void disabledConfigButtonAccordingToJonasBaseField() {
        if (this.runningJonasFieldFromDiskFrame.getUseJonasBase().isSelected()) {
            if (Utility.isValidJOnASBaseFolder(new File(this.runningJonasFieldFromDiskFrame.getJonasBase().getText()))
                    && this.runningJonasFieldFromDiskFrame.getUseJonasBase().isSelected()) {
                getWizard().getConfigButton().setVisible(false);
                File jBase = new File(this.runningJonasFieldFromDiskFrame.getJonasBase().getText());
                this.starter.setJonasBase(jBase);
                this.runningJonasFieldFromDiskFrame.getErrorLabel().setText("");
            } else {
                getWizard().getConfigButton().setVisible(false);
                this.runningJonasFieldFromDiskFrame.getErrorLabel().setText("Error: " + "Invalid JOnAS Base folder");
            }
        }
    }

    /**
     * Browse JOnAS Root folder.
     */
    private void browseJonasRoot() {
        JFileChooser fc = new JFileChooser();
        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fc.setAcceptAllFileFilterUsed(false);
        if (fc.showOpenDialog(this.runningJonasFieldFromDiskFrame.getJonasRoot()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            this.runningJonasFieldFromDiskFrame.getJonasRoot().setText(file.getAbsolutePath());
            setNextButtonAccordingToJonasRootField();
        }
    }

    /**
     * Browse JOnAS Base folder.
     */
    private void browseJonasBase() {
        JFileChooser fc = new JFileChooser();
        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fc.setAcceptAllFileFilterUsed(false);
        if (fc.showOpenDialog(this.runningJonasFieldFromDiskFrame.getJonasBase()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            this.runningJonasFieldFromDiskFrame.getJonasBase().setText(file.getAbsolutePath());
            disabledConfigButtonAccordingToJonasBaseField();
        }
    }

    /**
     * Browse Java home folder.
     */
    private void browseJavaHome() {
        JFileChooser fc = new JFileChooser();
        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fc.setAcceptAllFileFilterUsed(false);
        if (fc.showOpenDialog(this.runningJonasFieldFromDiskFrame.getJavaHome()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            this.runningJonasFieldFromDiskFrame.getJavaHome().setText(file.getAbsolutePath());
            setNextButtonAccordingToJonasRootField();
        }
    }

    /**
     * Check the Jav home value.
     * @return true if folder Java Home is a valid Java home folder, false
     *         otherwise.
     */
    private boolean checkJavaHome() {
        if (this.runningJonasFieldFromDiskFrame.getJavaHome().getText().equals("")) {
            return false;
        } else if (!this.starter.isValidJavaHomeFolder(new File(this.runningJonasFieldFromDiskFrame.getJavaHome().getText()))) {
            return false;
        }
        return true;
    }

    /**
     * Initialize JOnAS Root (and Base) folder with the appropriate value.
     */
    private void initializeJonasFolder() {
        String root = this.starter.searchExistingJonasDir(true);
        if (root != null) {
            this.runningJonasFieldFromDiskFrame.getJonasRoot().setText(root);
            File jRoot = new File(this.runningJonasFieldFromDiskFrame.getJonasRoot().getText());
            this.starter.setJonasRoot(jRoot);
            this.runningJonasFieldFromDiskFrame.getErrorLabel().setText("");
            String base = this.starter.searchExistingJonasDir(false);
            if (base != null) {
                this.runningJonasFieldFromDiskFrame.getJonasBase().setText(base);
                File jBase = new File(this.runningJonasFieldFromDiskFrame.getJonasBase().getText());
                this.starter.setJonasBase(jBase);
                this.runningJonasFieldFromDiskFrame.getUseJonasBase().setSelected(true);
                this.runningJonasFieldFromDiskFrame
                        .showJonasBaseField(this.runningJonasFieldFromDiskFrame.getUseJonasBase().isSelected());
                this.runningJonasFieldFromDiskFrame.getErrorLabel().setText("");
            }
            setNextButtonAccordingToJonasRootField();
        }
    }

    /**
     * The action performed listener.
     * @param event the event to performed.
     */
    public void actionPerformed(final ActionEvent event) {
        if (event.getSource() == this.runningJonasFieldFromDiskFrame.getJbBrowseJonasRoot()) {
            browseJonasRoot();
        } else if (event.getSource() == this.runningJonasFieldFromDiskFrame.getJbBrowseJonasBase()) {
            browseJonasBase();
        } else if (event.getSource() == this.runningJonasFieldFromDiskFrame.getJbBrowseJavaHome()) {
            browseJavaHome();
        } else if (event.getSource() == this.runningJonasFieldFromDiskFrame.getUseJonasBase()) {
            this.runningJonasFieldFromDiskFrame.showJonasBaseField(this.runningJonasFieldFromDiskFrame.getUseJonasBase().isSelected());
            if (this.runningJonasFieldFromDiskFrame.getUseJonasBase().isSelected()) {
                this.runningJonasFieldFromDiskFrame.getJonasBase().setText("");
                disabledConfigButtonAccordingToJonasBaseField();
            } else {
                this.runningJonasFieldFromDiskFrame.getJonasBase().setText("");
            }
        }
    }

    /**
     * Action to do when a key is typed.
     * @param ke the key typed.
     */
    public void keyTyped(final KeyEvent ke) {
    }

    /**
     * Action to do when a key is pressed.
     * @param ke the key pressed.
     */
    public void keyPressed(final KeyEvent ke) {
    }

    /**
     * Action to do when a key is released.
     * @param ke the key released.
     */
    public void keyReleased(final KeyEvent ke) {
        if (ke.getSource() == this.runningJonasFieldFromDiskFrame.getJonasRoot()) {
            setNextButtonAccordingToJonasRootField();
        } else if (ke.getSource() == this.runningJonasFieldFromDiskFrame.getJonasBase()) {
            disabledConfigButtonAccordingToJonasBaseField();
        } else if (ke.getSource() == this.runningJonasFieldFromDiskFrame.getJavaHome()) {
            setNextButtonAccordingToJonasRootField();
        }
    }
}
