/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GuiFullRunningJonasFieldFromEmbeddedFrameDescriptor.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.guifullstarter;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;

import javax.swing.JFileChooser;

import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.frame.RunningJonasFieldFromEmbeddedFrame;
import org.ow2.jonas.autostart.fullwizard.FullWizardPanelDescriptor;
import org.ow2.jonas.autostart.starterfull.FullStarter;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class instanciates RunningJonasFieldFromEmbeddedFrameDescriptor.
 * @author skabore
 */
public class GuiFullRunningJonasFieldFromEmbeddedFrameDescriptor extends FullWizardPanelDescriptor implements ActionListener,
        KeyListener {

    /**
     * The identifier.
     */
    public static final String IDENTIFIER = "RUN_JONAS_FROM_EMBEDDED_PANEL_FULL_STARTER";

    /**
     * The file separator.
     */
    private static final String FILE_SEPARATOR = System.getProperty("file.separator");

    /**
     * The RunningJonasFieldFromEmbeddedFrame.
     */
    private RunningJonasFieldFromEmbeddedFrame runningJonasFieldFromEmbeddedFrame;

    /**
     * The starter librairy.
     */
    private FullStarter starter;

    /**
     * The builder librairy.
     */
    private Builder builder;

    /**
     * The default constructor.
     * @param starter the starter librairy
     * @param builder the builder librairy
     * @param steps, the required steps to create a starter
     * @param stepId, the sted number
     */
    public GuiFullRunningJonasFieldFromEmbeddedFrameDescriptor(final FullStarter starter, final Builder builder,
            final int steps, final int stepId, final boolean reconf) {
        this.starter = starter;
        this.builder = builder;
        runningJonasFieldFromEmbeddedFrame = new RunningJonasFieldFromEmbeddedFrame(reconf, steps, stepId);
        runningJonasFieldFromEmbeddedFrame.getJonasRoot().setText(
                System.getProperty("user.dir") + FILE_SEPARATOR + starter.getJonasRootFolder());
        runningJonasFieldFromEmbeddedFrame.getJonasBase().setText(
                System.getProperty("user.dir") + FILE_SEPARATOR + starter.getJonasBaseFolder());
        runningJonasFieldFromEmbeddedFrame.addActionListener(this);
        runningJonasFieldFromEmbeddedFrame.addKeyListener(this);
        setPanelDescriptorIdentifier(IDENTIFIER);
        setPanelComponent(runningJonasFieldFromEmbeddedFrame);
    }

    /**
     * @return starter.
     */
    public FullStarter getStarter() {
        return starter;
    }

    /**
     * @return builder.
     */
    public Builder getBuilder() {
        return builder;
    }

    /**
     * Set the builder.
     * @param builder the builder new value
     */
    public void setBuilder(final Builder builder) {
        this.builder = builder;
    }

    /**
     * @return the next panel.
     */
    @Override
    public Object getNextPanelDescriptor() {
        getWizard().getStopButton().setVisible(false);
        getWizard().setCurrentFrame(10);
        return GuiFullApplicationFrameDescriptor.IDENTIFIER;
    }

    /**
     * @return the back panel.
     */
    @Override
    public Object getBackPanelDescriptor() {
        getWizard().getStopButton().setVisible(false);
        getWizard().getConfigButton().setVisible(false);
        return GuiFullRunningJonasOptionFrameDescriptor.IDENTIFIER;
    }

    /**
     * @return the config panel.
     */
    @Override
    public Object getConfigPanelDescriptor() {
        getWizard().getStopButton().setVisible(false);
        getWizard().getConfigButton().setVisible(false);
        return "ConfiguratorFrameDescriptor.IDENTIFIER";
    }

    /**
     * Set the next button text.
     */
    @Override
    public void displayingPanel() {
        getWizard().getNextButton().setText("Next");
    }

    /**
     * Check if the next button must be enabled.
     */
    @Override
    public void aboutToDisplayPanel() {
        setNextButtonAccordingToJonasRootField();
    }

    /**
     * Set the next button enabled according fields values.
     */
    private void setNextButtonAccordingToJonasRootField() {
        if (!checkJavaHome()) {
            getWizard().setNextFinishButtonEnabled(false);
            getWizard().getConfigButton().setVisible(false);
            runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText("" + "JAVA_HOME folder seems invalid.");
            getWizard().setNextFinishButtonEnabled(false);
        } else {
            File javaHome = new File(runningJonasFieldFromEmbeddedFrame.getJavaHome().getText());
            starter.setJavaHome(javaHome);
            runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText("");
            String rootInstallDir = runningJonasFieldFromEmbeddedFrame.getJonasRoot().getText();
            if (rootInstallDir.substring(rootInstallDir.lastIndexOf(FILE_SEPARATOR) + 1).equalsIgnoreCase(
                    starter.getJonasRootFolder())) {
                rootInstallDir = rootInstallDir.substring(0, rootInstallDir.lastIndexOf(FILE_SEPARATOR));
            }
            if (new File(rootInstallDir).isDirectory()) {
                if (new File(rootInstallDir).canWrite()) {
                    starter.setJonasRootInstallLocation(new File(rootInstallDir));
                    getWizard().setNextFinishButtonEnabled(true);
                    getWizard().getNextButton().setText("Start");
                    getWizard().getNextButton().setMnemonic(KeyEvent.VK_A);
                    getWizard().getConfigButton().setVisible(false);
                    runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText("");
                    starter.setUseEmbeddedJonas(true);
                    disabledConfigButtonAccordingToJonasBaseField();
                    checkInstallDirectory();
                } else {
                    runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText(
                            "This" + " directory can not be written." + " Please choose an other directory !");
                    getWizard().setNextFinishButtonEnabled(false);
                    getWizard().getConfigButton().setVisible(false);
                }
            } else {
                runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText(rootInstallDir + " is not valid !");
                getWizard().setNextFinishButtonEnabled(false);
            }
        }
    }

    /**
     * Disable config button according JOnAS Base fields.
     */
    private void disabledConfigButtonAccordingToJonasBaseField() {
        if (runningJonasFieldFromEmbeddedFrame.getUseJonasBase().isSelected()) {
            String baseInstallDir = runningJonasFieldFromEmbeddedFrame.getJonasBase().getText();
            if (baseInstallDir.substring(baseInstallDir.lastIndexOf(FILE_SEPARATOR) + 1).equalsIgnoreCase(
                    starter.getJonasBaseFolder())) {
                baseInstallDir = baseInstallDir.substring(0, baseInstallDir.lastIndexOf(FILE_SEPARATOR));
            }
            if (new File(baseInstallDir).isDirectory()) {
                if (new File(baseInstallDir).canWrite()) {
                    runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText("");
                    getWizard().getConfigButton().setVisible(false);
                    starter.setJonasBaseInstallLocation(new File(baseInstallDir));
                } else {
                    runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText(
                            "This" + " directory can not be written." + " Please choose an other directory !");
                    getWizard().getConfigButton().setVisible(false);
                }
            } else {
                runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText(baseInstallDir + " is not valid !");
            }
        }
    }

    /**
     * Browse JOnAS Root folder.
     */
    private void browseJonasRoot() {
        JFileChooser fc = new JFileChooser();
        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fc.setAcceptAllFileFilterUsed(false);
        if (fc.showOpenDialog(runningJonasFieldFromEmbeddedFrame.getJonasRoot()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            runningJonasFieldFromEmbeddedFrame.getJonasRoot().setText(file.getAbsolutePath());
            setNextButtonAccordingToJonasRootField();
        }
    }

    /**
     * Browse JOnAS Base folder.
     */
    private void browseJonasBase() {
        JFileChooser fc = new JFileChooser();
        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fc.setAcceptAllFileFilterUsed(false);
        if (fc.showOpenDialog(runningJonasFieldFromEmbeddedFrame.getJonasBase()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            runningJonasFieldFromEmbeddedFrame.getJonasBase().setText(file.getAbsolutePath());
            disabledConfigButtonAccordingToJonasBaseField();
        }
    }

    /**
     * Browse Java home folder.
     */
    private void browseJavaHome() {
        JFileChooser fc = new JFileChooser();
        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fc.setAcceptAllFileFilterUsed(false);
        if (fc.showOpenDialog(runningJonasFieldFromEmbeddedFrame.getJavaHome()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            runningJonasFieldFromEmbeddedFrame.getJavaHome().setText(file.getAbsolutePath());
            setNextButtonAccordingToJonasRootField();
        }
    }

    /**
     * Check the Jav home value.
     * @return true if folder Java Home is a valid Java home folder, false
     *         otherwise.
     */
    private boolean checkJavaHome() {
        if (runningJonasFieldFromEmbeddedFrame.getJavaHome().getText().equals("")) {
            return false;
        } else {
            return starter.isValidJavaHomeFolder(new File(runningJonasFieldFromEmbeddedFrame.getJavaHome().getText()));
        }
    }

    /**
     * Check if the install directory does not contain JOnAS_Root or JOnAS_Base
     * folder.
     */
    private void checkInstallDirectory() {
        String rootInstallDir = runningJonasFieldFromEmbeddedFrame.getJonasRoot().getText();
        if (!rootInstallDir.substring(rootInstallDir.lastIndexOf(FILE_SEPARATOR) + 1).equalsIgnoreCase(
                starter.getJonasRootFolder())) {
            if (Utility.containFile(new File(rootInstallDir), starter.getJonasRootFolder())) {
                String msg = starter.getJonasRootFolder() + " already exists in " + rootInstallDir
                        + ". It will be overwritten if you continue.";
                runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText(msg);
            }
            rootInstallDir = rootInstallDir + FILE_SEPARATOR + starter.getJonasRootFolder();
            runningJonasFieldFromEmbeddedFrame.getJonasRoot().setText(rootInstallDir);
            if (runningJonasFieldFromEmbeddedFrame.getUseJonasBase().isSelected()) {
                String baseInstallDir = runningJonasFieldFromEmbeddedFrame.getJonasBase().getText();
                if (!baseInstallDir.substring(baseInstallDir.lastIndexOf(FILE_SEPARATOR) + 1).equalsIgnoreCase(
                        starter.getJonasBaseFolder())
                        && runningJonasFieldFromEmbeddedFrame.getUseJonasBase().isSelected()) {
                    if (Utility.containFile(new File(baseInstallDir), starter.getJonasBaseFolder())) {
                        String msg = starter.getJonasBaseFolder() + " already exists in " + baseInstallDir
                                + ". It will be overwritten if you continue.";
                        runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText(msg);
                    }
                    baseInstallDir = baseInstallDir + FILE_SEPARATOR + starter.getJonasBaseFolder();
                    runningJonasFieldFromEmbeddedFrame.getJonasBase().setText(baseInstallDir);
                }
            }
        } else {
            runningJonasFieldFromEmbeddedFrame.getErrorLabel().setText("");
        }
    }

    /**
     * The action performed listener.
     * @param event the event to performed.
     */
    public void actionPerformed(final ActionEvent event) {
        if (event.getSource() == runningJonasFieldFromEmbeddedFrame.getJbBrowseJonasRoot()) {
            browseJonasRoot();
        } else if (event.getSource() == runningJonasFieldFromEmbeddedFrame.getJbBrowseJonasBase()) {
            browseJonasBase();
        } else if (event.getSource() == runningJonasFieldFromEmbeddedFrame.getJbBrowseJavaHome()) {
            browseJavaHome();
        } else if (event.getSource() == runningJonasFieldFromEmbeddedFrame.getUseJonasBase()) {
            runningJonasFieldFromEmbeddedFrame.showJonasBaseField(runningJonasFieldFromEmbeddedFrame.getUseJonasBase()
                    .isSelected());
            runningJonasFieldFromEmbeddedFrame.getJonasBase().setText(
                    System.getProperty("user.dir") + FILE_SEPARATOR + starter.getJonasBaseFolder());
            if (runningJonasFieldFromEmbeddedFrame.getUseJonasBase().isSelected()) {
                builder.setUseJonasFullStarter(true);
                disabledConfigButtonAccordingToJonasBaseField();
            } else {
                builder.setUseJonasFullStarter(false);
            }
        }
    }

    /**
     * Action to do when a key is typed.
     * @param ke the key typed.
     */
    public void keyTyped(final KeyEvent ke) {
    }

    /**
     * Action to do when a key is pressed.
     * @param ke the key pressed.
     */
    public void keyPressed(final KeyEvent ke) {
    }

    /**
     * Action to do when a key is released.
     * @param ke the key released.
     */
    public void keyReleased(final KeyEvent ke) {
        if (ke.getSource() == runningJonasFieldFromEmbeddedFrame.getJonasRoot()) {
            setNextButtonAccordingToJonasRootField();
        } else if (ke.getSource() == runningJonasFieldFromEmbeddedFrame.getJonasBase()) {
            disabledConfigButtonAccordingToJonasBaseField();
        } else if (ke.getSource() == runningJonasFieldFromEmbeddedFrame.getJavaHome()) {
            setNextButtonAccordingToJonasRootField();
        }
    }
}
