/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GuiFullStarterFrameDescriptor.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.guifullstarter;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.security.ProtectionDomain;

import org.ow2.jonas.autostart.fullwizard.FullWizardPanelDescriptor;
import org.ow2.jonas.autostart.starter.StarterException;
import org.ow2.jonas.autostart.starterfull.FullStarter;
import org.ow2.jonas.autostart.utility.Output;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class instantiates StarterFrame.
 * @author skabore
 */
public class GuiFullStarterFrameDescriptor extends FullWizardPanelDescriptor implements ActionListener {

    /**
     * The domaine.
     */
    private final ProtectionDomain domaine;

    /**
     * The starter librairy.
     */
    private final FullStarter starter;

    /**
     * The identifier.
     */
    public static final String IDENTIFIER = "STARTER_PANEL_FULL_STARTER";

    /**
     * The StarterFrame.
     */
    private final GuiFullStarterFrame starterFrame;

    /**
     * The constructor.
     * @param domaine the domaine
     * @param starter the starter librairy
     * @param steps, the required steps to create a starter
     */
    public GuiFullStarterFrameDescriptor(final ProtectionDomain domaine, final FullStarter starter, final int steps) {
        this.domaine = domaine;
        this.starter = starter;
        starterFrame = new GuiFullStarterFrame(starter, steps);
        this.starter.setOutput(new Output(starterFrame.getOutputTextArea()));
        starterFrame.setStarter(this.starter);
        starterFrame.addActionListener(this);
        setPanelDescriptorIdentifier(IDENTIFIER);
        setPanelComponent(starterFrame);
    }

    /**
     * @return starter.
     */
    public FullStarter getStarter() {
        return starter;
    }

    /**
     * @return the next panel.
     */
    @Override
    public Object getNextPanelDescriptor() {
        getWizard().getStopButton().setVisible(true);
        if (starter.jonasIsRunning()) {
            getWizard().getStopButton().setEnabled(false);
        }
        getWizard().getConfigButton().setVisible(false);
        return IDENTIFIER;
    }

    /**
     * @return the back panel.
     */
    @Override
    public Object getBackPanelDescriptor() {
        getWizard().getStopButton().setVisible(false);
        getWizard().getConfigButton().setVisible(false);
        return GuiFullApplicationFrameDescriptor.IDENTIFIER;
    }

    /**
     * Check if the next button must be enabled.
     */
    @Override
    public void aboutToDisplayPanel() {
        getWizard().setNextFinishButtonEnabled(false);
    }

    /**
     * Launch the start process.
     */
    @Override
    public void displayingPanel() {
        starter.setOutput(new Output(null, starterFrame.getOutputTextArea()));
        // starter.setVerboseMode(true);
        starter.setVerboseMode(false);
        starter.getOutput().write("");
        new Thread() {
            @Override
            public void run() {
                if (!starter.jonasIsRunning()) {
                    Cursor cursor = new Cursor(Cursor.WAIT_CURSOR);
                    starterFrame.setCursor(cursor);
                    getWizard().getStopButton().setEnabled(false);
                    getWizard().getNextButton().setEnabled(false);
                    getWizard().setBackButtonEnabled(false);
                    String path = domaine.getCodeSource().getLocation().getFile();
                    if (Utility.getPlatform().equalsIgnoreCase("windows")) {
                        path = path.replace("/", "\\");
                        path = path.replace("%20", " ");
                        path = path.substring(1);
                    }
                    starterFrame.setProgressText("");
                    starter.getOutput().write("");
                    starterFrame.setProgressValue(10);
                    starter.createTmpWorkDirectory();
                    if (starter.getUseEmbeddedJonas()) {
                        starterFrame.setProgressText("Extracting JOnAS" + " server...... Please wait !");
                        starter.getOutput().write("Extracting JOnAS" + " server...");
                        starter.setStarterLocation(new File(path));
                        try {
                            starter.extractJonas();
                        } catch (StarterException e) {
                            starter.getOutput().write("Error while extracting JOnAS server : " + e.getMessage());
                        }

                        if (getWizard().getConfiguration() != null && Utility.isFullJonasFolder(starter.getJonasRoot())) {
                            starterFrame.setProgressValue(25);
                            starter.setConfiguration(getWizard().getConfiguration());
                            starter.applyReconfiguration();
                        }
                        starter.setUseEmbeddedJonas(false);
                    }

                    starter.deployApplication();
                    starterFrame.setProgressValue(50);
                    starterFrame.setProgressText("Starting JOnAS server..." + " Please wait !");

                    starterFrame.setProgressValue(75);

                    starter.getOutput().write("");
                    starter.getOutput().write("Starting JOnAS server...");
                    starter.startJonas();
                    starterFrame.setProgressText("Running" + " JOnAS server completed !");
                    starter.getOutput().write("Running" + " JOnAS server completed !");
                    starterFrame.setProgressValue(100);
                    getWizard().setNextFinishButtonEnabled(false);
                    getWizard().getStopButton().setEnabled(true);
                    getWizard().setBackButtonEnabled(true);
                    cursor = new Cursor(Cursor.DEFAULT_CURSOR);
                    starterFrame.setCursor(cursor);
                    Utility.deleteDirectory(starter.getWorkdirectory());
                } else {
                    getWizard().showMessage("JOnAS server is already" + " running.\n" + " Stop it before starting !");
                    getWizard().getStopButton().setEnabled(true);
                }
            }
        }.start();
    }

    /**
     * The action performed listener.
     * @param event the event to performed.
     */
    public void actionPerformed(final ActionEvent event) {
        if (event.getSource() == starterFrame.getShowDetailButton()) {
            starterFrame.showDetail();
        }
    }
}
